==> 0.0.10.tar.gz: lib/classes/Responses/reader_read_res.dart <==
import 'dart:convert';

import 'package:artemis_cupps/artemis_cupps.dart';
import 'package:encrypt/encrypt.dart';
import 'package:xml/xml.dart';
import '../classes.dart';

class ReaderReadResponse extends CuppsBase {
  final String result;
  final List<ReaderData> dataList;

  ReaderReadResponse({
    required int messageID,
    required String messageName,
    required this.result,
    required this.dataList,
  }) : super(messageID: messageID, messageName: messageName);

  factory ReaderReadResponse.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageName")
        .value;
    String messageID = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageID")
        .value;
    XmlElement resElement = document.findAllElements(messageName).first;
    String result = resElement.attributes
        .firstWhere((p0) => p0.name.toString() == "result")
        .value;

    List<XmlElement> readerElementList =
        document.rootElement.findAllElements("readerData").toList();

    List<ReaderData> readerDataList = readerElementList
        .map((e) => ReaderData.fromXML(e.toXmlString()))
        .toList();
    return ReaderReadResponse(
      messageName: messageName,
      messageID: int.parse(messageID),
      result: result,
      dataList: readerDataList,
    );
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <$messageName result="$result">
          ${dataList.map((e) => e.toXML()).join()}
        </$messageName>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName: $result";
  }
}

class ReaderData {
  final Data data;

  ReaderData({required this.data});

  factory ReaderData.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    XmlElement? dataElement = document.firstChild!.childElements
        .firstWhere((element) => element.name.toString().endsWith("Data"));

    Data data = Data();
    if (dataElement.name.toString().startsWith("bc")) {
      data = BcData.fromXML(dataElement.toXmlString());
    } else if (dataElement.name.toString().startsWith("ms")) {
      data = MsData.fromXML(dataElement.toXmlString());
    } else if (dataElement.name.toString().startsWith("oc")) {
      data = OcData.fromXML(dataElement.toXmlString());
    } else if (dataElement.name.toString().startsWith("sd")) {
      data = SdData.fromXML(dataElement.toXmlString());
    }

    return ReaderData(data: data);
  }

  String toXML() {
    return '''<readerData>${data.toXML()}</readerData>''';
  }
}

class Data {
  String toXML() {
    // TODO: implement toString
    return "";
  }
}

class BcData extends Data {
  final String bcTypeCode;
  final String readStatus;
  final String? dataSTR;

  BcData({
    required this.bcTypeCode,
    required this.readStatus,
    required this.dataSTR,
  });

  factory BcData.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String? str = document.firstChild?.text;
    String bcTypeCode = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "bcTypeCode")
        .value;
    String readStatus = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "readStatus")
        .value;

    return BcData(bcTypeCode: bcTypeCode, readStatus: readStatus, dataSTR: str);
  }

  @override
  String toXML() {
    return '''<bcData bcTypeCode="$bcTypeCode" readStatus="$readStatus">${dataSTR ?? ""}</bcData>''';
  }

  String get decodedValue {
    String val = dataSTR == null ? "" : utf8.decode(base64Decode(dataSTR!));
    // val = val.replaceFirst(bcTypeCode, "");
    return val;
  }
}

class MsData extends Data {
  final String msType;
  final String readStatus;
  final List<MSTrackData> msTrackData;

  MsData({
    required this.msType,
    required this.readStatus,
    required this.msTrackData,
  });

  factory MsData.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    List<XmlElement> trackList =
        document.findAllElements("msTrackData").toList();

    String msType = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "msType",
            orElse: orElseEmpty)
        .value;
    String readStatus = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "readStatus")
        .value;
    List<MSTrackData> dataList =
        trackList.map((e) => MSTrackData.fromXML(e.toXmlString())).toList();
    return MsData(
        msType: msType, readStatus: readStatus, msTrackData: dataList);
  }

  @override
  String toXML() {
    return '''<msData msType="$msType" readStatus="$readStatus">${msTrackData.map((e) => e.toXML()).join()}</msData>''';
  }

  @override
  String toString() {
    String trackData = msTrackData.map((e) => e.blockData.decrypt()).join("\n");
    // TODO: implement toString
    return trackData;
  }
}

class MSTrackData {
  final String trackID;
  final BlockData blockData;

  MSTrackData({
    required this.trackID,
    required this.blockData,
  });

  factory MSTrackData.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    XmlElement? dataElement = document.firstChild!.getElement("blockData");

    String trackID = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "trackID")
        .value;
    BlockData data = BlockData.fromXML(dataElement!.toXmlString());
    return MSTrackData(trackID: trackID, blockData: data);
  }

  String toXML() {
    return '''<msTrackData trackID="$trackID">${blockData.toXML()}</msTrackData>''';
  }

  String decrypt(){
    return blockData.decrypt();
  }

}

class BlockData {
  final String blockID;
  final String value;

  BlockData({
    required this.blockID,
    required this.value,
  });

  factory BlockData.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String value = (document.firstChild?.text ?? "").replaceAll("<", "&lt;");
    String blockID = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "blockID")
        .value;
    return BlockData(blockID: blockID, value: value);
  }

  String toXML() {
    return '''<blockData blockID="$blockID">$value</blockData>''';
  }

  String decrypt() {
    try {
      final key = Key.fromUtf8("4D6B225A27694BA4");
      // final key = Key.fromUtf8(CUPPS().deviceToken!);
      final iv = IV.fromBase64("4D6B225A27694BA4");
      final encrypter = Encrypter(AES(key, mode: AESMode.ecb,padding: null));
      Encrypted encrypted = Encrypted.fromBase64(value);
      // Encrypted encrypted = Encrypted.fromUtf8("TEST");

      final decrypted = encrypter.decrypt(encrypted,iv: iv);
      return decrypted;
      return encrypted.base64;
    }catch (e){
      throw e;
    }

  }

}

class OcData extends Data {
  final String readStatus;
  final List<OcTrackData> ocTrackData;

  OcData({
    required this.readStatus,
    required this.ocTrackData,
  });

  factory OcData.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    List<XmlElement> trackList =
        document.findAllElements("ocTrackData").toList();

    String readStatus = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "readStatus")
        .value;
    List<OcTrackData> dataList =
        trackList.map((e) => OcTrackData.fromXML(e.toXmlString())).toList();
    return OcData(readStatus: readStatus, ocTrackData: dataList);
  }

  @override
  String toXML() {
    return '''<ocData readStatus="$readStatus">${ocTrackData.map((e) => e.toXML()).join()}</ocData>''';
  }

  @override
  String toString() {
    String val = ocTrackData.map((e) => e.value).join("");
    return val;
  }

  Map<String,dynamic> getParsedDate(){
    ///"PMCHEPHILLIPS<<AGNES<<<<<<<<<<<<<<<<<<<<<<<<X3600984<3CHE5011106F2404071<<<<<<<<<<<<<<<4"
    String type = "Unknown";

    String val = ocTrackData.map((e) => e.value).join("");
    String firstChar = val.substring(0,1);
    if(firstChar=="p"){
      type = "Passport";
    }else if(firstChar =="V"){
      type = "Visa";
    }else if(firstChar == "I"||firstChar == "A"||firstChar == "C"){
      type = "Official Travel Document";
    }




    return {
      "Type":type,
      "RawData":val
    };
  }
}

class OcTrackData {
  final String trackID;
  final String value;

  OcTrackData({
    required this.trackID,
    required this.value,
  });

  factory OcTrackData.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String value = document.firstChild?.text ?? "";
    String trackID = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "trackID")
        .value;
    return OcTrackData(trackID: trackID, value: value);
  }

  String toXML() {
    return '''<ocTrackData trackID="$trackID">${value.replaceAll("<", "&lt;")}</ocTrackData>''';
  }
}

class SdData extends Data {
  final String weight;
  final String readStatus;

  SdData({
    required this.weight,
    required this.readStatus,
  });

  factory SdData.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String weight = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "weight")
        .value;
    String readStatus = document.firstChild!.attributes
        .firstWhere((element) => element.name.toString() == "readStatus")
        .value;
    return SdData(readStatus: readStatus, weight: weight);
  }

  @override
  String toXML() {
    return '''<sdData readStatus="$readStatus" weight="$weight"/>''';
  }
}

class BgData extends Data {
  final String value;
  BgData({
    required this.value,
  });
}

class BgResponse {
  final bool status;
  final String message;

  BgResponse({required this.status, required this.message});
}

==> 0.0.14.tar.gz: lib/classes/Responses/reader_read_res.dart <==
import 'dart:convert';
import 'dart:developer';

import 'package:artemis_cupps/artemis_cupps.dart';
import 'package:encrypt/encrypt.dart';
import 'package:intl/intl.dart';
import 'package:xml/xml.dart';
import '../classes.dart';

class ReaderReadResponse extends CuppsBase {
  final String result;
  final List<ReaderData> dataList;

  ReaderReadResponse({
    required int messageID,
    required String messageName,
    required this.result,
    required this.dataList,
  }) : super(messageID: messageID, messageName: messageName);

  factory ReaderReadResponse.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String messageName = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageName")
        .value;
    String messageID = document.rootElement.attributes
        .firstWhere((p0) => p0.name.toString() == "messageID")
        .value;
    XmlElement resElement = document
        .findAllElements(messageName)
        .first;
    String result = resElement.attributes
        .firstWhere((p0) => p0.name.toString() == "result")
        .value;

    List<XmlElement> readerElementList =
    document.rootElement.findAllElements("readerData").toList();

    List<ReaderData> readerDataList = readerElementList
        .map((e) => ReaderData.fromXML(e.toXmlString()))
        .toList();
    return ReaderReadResponse(
      messageName: messageName,
      messageID: int.parse(messageID),
      result: result,
      dataList: readerDataList,
    );
  }

  @override
  String toXML() {
    return '''
    <?xml version="1.0" encoding="utf-8"?>
    <cupps xmlns="$xmlns" messageID="$messageID" xmlns:xsi="$xsi" messageName="$messageName">
        <$messageName result="$result">
          ${dataList.map((e) => e.toXML()).join()}
        </$messageName>
    </cupps>
    '''
        .replaceAll("\n", "")
        .replaceAll("\t", "")
        .replaceAll("  ", "");
  }

  @override
  String toString() {
    return "$messageID : $messageName: $result";
  }
}

class ReaderData {
  final Data data;

  ReaderData({required this.data});

  factory ReaderData.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    XmlElement? dataElement = document.firstChild!.childElements
        .firstWhere((element) => element.name.toString().endsWith("Data"));

    Data data = Data();
    if (dataElement.name.toString().startsWith("bc")) {
      data = BcData.fromXML(dataElement.toXmlString());
    } else if (dataElement.name.toString().startsWith("ms")) {
      data = MsData.fromXML(dataElement.toXmlString());
    } else if (dataElement.name.toString().startsWith("oc")) {
      data = OcData.fromXML(dataElement.toXmlString());
    } else if (dataElement.name.toString().startsWith("sd")) {
      data = SdData.fromXML(dataElement.toXmlString());
    }

    return ReaderData(data: data);
  }

  String toXML() {
    return '''<readerData>${data.toXML()}</readerData>''';
  }
}

class Data {
  String toXML() {
    // TODO: implement toString
    return "";
  }
}

class BcData extends Data {
  final String bcTypeCode;
  final String readStatus;
  final String? dataSTR;

  BcData({
    required this.bcTypeCode,
    required this.readStatus,
    required this.dataSTR,
  });

  factory BcData.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String? str = document.firstChild?.text;
    String bcTypeCode = document.firstChild!
        .attributes
        .firstWhere((element) => element.name.toString() == "bcTypeCode")
        .value;
    String readStatus = document.firstChild!
        .attributes
        .firstWhere((element) => element.name.toString() == "readStatus")
        .value;

    return BcData(bcTypeCode: bcTypeCode, readStatus: readStatus, dataSTR: str);
  }

  @override
  String toXML() {
    return '''<bcData bcTypeCode="$bcTypeCode" readStatus="$readStatus">${dataSTR ?? ""}</bcData>''';
  }

  String get decodedValue {
    String val = dataSTR == null ? "" : utf8.decode(base64Decode(dataSTR!));
    // val = val.replaceFirst(bcTypeCode, "");
    return val;
  }
}

class MsData extends Data {
  final String msType;
  final String readStatus;
  final List<MSTrackData> msTrackData;

  MsData({
    required this.msType,
    required this.readStatus,
    required this.msTrackData,
  });

  factory MsData.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    List<XmlElement> trackList =
    document.findAllElements("msTrackData").toList();

    String msType = document.firstChild!
        .attributes
        .firstWhere((element) => element.name.toString() == "msType",
        orElse: orElseEmpty)
        .value;
    String readStatus = document.firstChild!
        .attributes
        .firstWhere((element) => element.name.toString() == "readStatus")
        .value;
    List<MSTrackData> dataList =
    trackList.map((e) => MSTrackData.fromXML(e.toXmlString())).toList();
    return MsData(
        msType: msType, readStatus: readStatus, msTrackData: dataList);
  }

  @override
  String toXML() {
    return '''<msData msType="$msType" readStatus="$readStatus">${msTrackData.map((e) => e.toXML()).join()}</msData>''';
  }

  @override
  String toString() {
    String trackData = msTrackData.map((e) => e.blockData.decrypt()).join("\n");
    // TODO: implement toString
    return trackData;
  }
}

class MSTrackData {
  final String trackID;
  final BlockData blockData;

  MSTrackData({
    required this.trackID,
    required this.blockData,
  });

  factory MSTrackData.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    XmlElement? dataElement = document.firstChild!.getElement("blockData");

    String trackID = document.firstChild!
        .attributes
        .firstWhere((element) => element.name.toString() == "trackID")
        .value;
    BlockData data = BlockData.fromXML(dataElement!.toXmlString());
    return MSTrackData(trackID: trackID, blockData: data);
  }

  String toXML() {
    return '''<msTrackData trackID="$trackID">${blockData.toXML()}</msTrackData>''';
  }

  String decrypt() {
    return blockData.decrypt();
  }

}

class BlockData {
  final String blockID;
  final String value;

  BlockData({
    required this.blockID,
    required this.value,
  });

  factory BlockData.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String value = (document.firstChild?.text ?? "").replaceAll("<", "&lt;");
    String blockID = document.firstChild!
        .attributes
        .firstWhere((element) => element.name.toString() == "blockID")
        .value;
    return BlockData(blockID: blockID, value: value);
  }

  String toXML() {
    return '''<blockData blockID="$blockID">$value</blockData>''';
  }

  String decrypt() {
    try {
      // final key = Key.fromUtf8("4D6B225A27694BA4");
      // final iv = IV.fromBase64("4D6B225A27694BA4");
      final key = Key.fromUtf8(CUPPS().deviceToken!);
      final iv = IV.fromBase64(CUPPS().deviceToken!);
      final encrypter = Encrypter(AES(key, mode: AESMode.ecb, padding: null));
      Encrypted encrypted = Encrypted.fromBase64(value);
      // Encrypted encrypted = Encrypted.fromUtf8("TEST");

      final decrypted = encrypter.decrypt(encrypted, iv: iv);
      return decrypted;
      return encrypted.base64;
    } catch (e) {
      throw e;
    }
  }

}

class OcData extends Data {
  final String readStatus;
  final List<OcTrackData> ocTrackData;

  OcData({
    required this.readStatus,
    required this.ocTrackData,
  });

  factory OcData.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    List<XmlElement> trackList =
    document.findAllElements("ocTrackData").toList();

    String readStatus = document.firstChild!
        .attributes
        .firstWhere((element) => element.name.toString() == "readStatus")
        .value;
    List<OcTrackData> dataList =
    trackList.map((e) => OcTrackData.fromXML(e.toXmlString())).toList();
    return OcData(readStatus: readStatus, ocTrackData: dataList);
  }

  @override
  String toXML() {
    return '''<ocData readStatus="$readStatus">${ocTrackData.map((e) => e.toXML()).join()}</ocData>''';
  }

  @override
  String toString() {
    String val = ocTrackData.map((e) => e.value).join("");
    return val;
  }

  Map<String, dynamic> getParsedDate() {
    ///"PMCHEPHILLIPS<<AGNES<<<<<<<<<<<<<<<<<<<<<<<<X3600984<3CHE5011106F2404071<<<<<<<<<<<<<<<4"
    try {
      String val = ocTrackData.map((e) => e.value).join("");
    String type = "Unknown";
    String passType = "Normal";
    String country = "";
    String firstname = "";
    String lastname = "";
    String passportNumber = "";
    String nationality = "";
    String birthDate = "";
    String sex = "Unspecified";
    String expireDate = "";
    String personalNumber = "";


    String firstRow = val.substring(0, 44);
    String secondRow = val.substring(0, 88).replaceFirst(firstRow, "");

    String firstChar = val.substring(0, 1);
    if (firstChar.toLowerCase() == "p") {
      type = "Passport";
    } else if (firstChar == "V") {
      type = "Visa";
    } else if (firstChar == "I" || firstChar == "A" || firstChar == "C") {
      type = "Official Travel Document";
    }

    String secondChar = val.substring(1, 2);
    if (secondChar != "<") {
      passType = secondChar;
    }

    country = val.substring(2, 5);

    String nameSection = val.substring(5, 44);
    String surname = nameSection
        .split("<<")
        .first;
    String givenNamePart = nameSection.replaceFirst("$surname<<", "");
    String givenName = givenNamePart.split("<").join(" ");
    lastname = surname;
    firstname = givenName.replaceAll("  ", "").replaceAll("<", "");;

    passportNumber = secondRow.substring(0, 9);
    nationality = secondRow.substring(10, 13);
    birthDate = secondRow.substring(13, 19);
    sex = secondRow.substring(20, 21) != "<" ? secondRow.substring(20, 21) : sex;
    expireDate = secondRow.substring(21, 27);
    personalNumber = secondRow.substring(28, 42).replaceAll("<", "");

    birthDate = birthDate.substring(0,2)+"-"+birthDate.substring(2,4)+"-"+birthDate.substring(4,6);
    expireDate = expireDate.substring(0,2)+"-"+expireDate.substring(2,4)+"-"+expireDate.substring(4,6);


      DateTime birthDateTime = DateFormat("yy-MM-dd").parse(birthDate);
      DateTime expireDateTime = DateFormat("yy-MM-dd").parse(expireDate);

      birthDate =DateFormat("yyyy-MM-dd").format(birthDateTime);
      expireDate =DateFormat("yyyy-MM-dd").format(expireDateTime);
    return {
      "Type": type,
      "FirstName": firstname,
      "LastName": lastname,
      "PassportType": passType,
      "Country": country,
      "PassportNumber": passportNumber,
      "Nationality": nationality,
      "BirthDate": birthDate,
      "Sex": sex,
      "ExpireDate": expireDate,
      "PersonalNumber": personalNumber,
      "RawData": val,
    };
    }catch (e) {
      String val = ocTrackData.map((e) => e.value).join("");
      log("$e");
      return {
        "RawData":val,
        "Exception":"$e"
      };
    }



  }
}

class OcTrackData {
  final String trackID;
  final String value;

  OcTrackData({
    required this.trackID,
    required this.value,
  });

  factory OcTrackData.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);
    String value = document.firstChild?.text ?? "";
    String trackID = document.firstChild!
        .attributes
        .firstWhere((element) => element.name.toString() == "trackID")
        .value;
    return OcTrackData(trackID: trackID, value: value);
  }

  String toXML() {
    return '''<ocTrackData trackID="$trackID">${value.replaceAll("<", "&lt;")}</ocTrackData>''';
  }
}

class SdData extends Data {
  final String weight;
  final String readStatus;

  SdData({
    required this.weight,
    required this.readStatus,
  });

  factory SdData.fromXML(String xmlSTR) {
    final document = XmlDocument.parse(xmlSTR);

    String weight = document.firstChild!
        .attributes
        .firstWhere((element) => element.name.toString() == "weight")
        .value;
    String readStatus = document.firstChild!
        .attributes
        .firstWhere((element) => element.name.toString() == "readStatus")
        .value;
    return SdData(readStatus: readStatus, weight: weight);
  }

  @override
  String toXML() {
    return '''<sdData readStatus="$readStatus" weight="$weight"/>''';
  }
}

class BgData extends Data {
  final String value;

  BgData({
    required this.value,
  });
}

class BgResponse {
  final bool status;
  final String message;

  BgResponse({required this.status, required this.message});
}
